<h3>Welcome</h3>
<p>An account has been created for you.</p>
<p>Please set your password using the <a href="{{ reset_link }}">following link</a>.</p>
<p>You will be redirected to the login screen afterwards.</p>

<p>-- The ETS team</p>
